@@ -7,11 +7,10 @@ import pytest
 from numpy.testing import assert_array_equal
 
 import asdf
-import asdf.testing.helpers
 from asdf import config_context, constants, get_config, schema, tagged, util, yamlutil
-from asdf._tests import _helpers as helpers
 from asdf.exceptions import AsdfConversionWarning, AsdfDeprecationWarning, AsdfWarning, ValidationError
 from asdf.extension import TagDefinition
+from asdf.testing.helpers import yaml_to_asdf
 
 
 @contextlib.contextmanager
@@ -93,7 +92,7 @@ not_tagged:
     """
     with asdf.config_context() as cfg:
         cfg.add_extension(ScalarExtension())
-        buff = helpers.yaml_to_asdf(yaml)
+        buff = yaml_to_asdf(yaml)
         with asdf.open(buff) as ff:
             assert isinstance(ff.tree["tagged"], Scalar)
             assert not isinstance(ff.tree["not_tagged"], Scalar)
@@ -103,13 +102,13 @@ not_tagged:
             assert ff.tree["not_tagged"] == "m"
 
 
-def test_read_json_schema():
+def test_read_json_schema(test_data_path):
     """Pytest to make sure reading JSON schemas succeeds.
 
     This was known to fail on Python 3.5 See issue #314 at
     https://github.com/asdf-format/asdf/issues/314 for more details.
     """
-    json_schema = helpers.get_test_data_path("example_schema.json")
+    json_schema = test_data_path / "example_schema.json"
     schema_tree = schema.load_schema(json_schema, resolve_references=True)
     schema.check_schema(schema_tree)
 
@@ -450,33 +449,40 @@ default: 42
 custom: !<{tag_uri}>
   foo
     """
-    buff = helpers.yaml_to_asdf(yaml)
+    buff = yaml_to_asdf(yaml)
     # This should cause a warning but not an error because without explicitly
     # providing an extension, our custom type will not be recognized and will
     # simply be converted to a raw type.
-    with pytest.warns(AsdfConversionWarning, match=tag_uri), asdf.open(buff):
-        pass
+    with pytest.warns(AsdfConversionWarning, match=tag_uri):
+        with asdf.open(buff) as af:
+            af["custom"]
 
     buff.seek(0)
     with config_context() as cfg:
         cfg.add_extension(CustomExtension())
         cfg.add_resource_mapping({schema_uri: tag_schema})
-        with pytest.raises(ValidationError, match=r".* is not of type .*"), asdf.open(
-            buff,
+        with (
+            pytest.raises(ValidationError, match=r".* is not of type .*"),
+            asdf.open(
+                buff,
+            ),
         ):
             pass
 
         # Make sure tags get validated inside of other tags that know
         # nothing about them.
         yaml = f"""
-    array: !core/ndarray-1.0.0
-      data: [0, 1, 2]
-      custom: !<{tag_uri}>
-        foo
+array: !core/ndarray-1.0.0
+  data: [0, 1, 2]
+  custom: !<{tag_uri}>
+    foo
         """
-        buff = helpers.yaml_to_asdf(yaml)
-        with pytest.raises(ValidationError, match=r".* is not of type .*"), asdf.open(
-            buff,
+        buff = yaml_to_asdf(yaml)
+        with (
+            pytest.raises(ValidationError, match=r".* is not of type .*"),
+            asdf.open(
+                buff,
+            ),
         ):
             pass
 
@@ -626,7 +632,7 @@ custom: !<http://nowhere.org/tags/custom/default-1.0.0>
   j:
     l: 362
         """
-        buff = helpers.yaml_to_asdf(yaml)
+        buff = yaml_to_asdf(yaml)
         with asdf.open(buff) as ff:
             assert "a" in ff.tree["custom"]
             assert ff.tree["custom"]["a"] == 42
@@ -738,7 +744,7 @@ one_of: !<{tag_uri}>
         cfg.add_extension(OneOfExtension())
         cfg.add_resource_mapping({schema_uri: tag_schema})
 
-        buff = helpers.yaml_to_asdf(yaml)
+        buff = yaml_to_asdf(yaml)
         with asdf.open(buff) as ff:
             assert ff["one_of"].value == "foo"
 
@@ -753,7 +759,7 @@ custom: !<tag:nowhere.org:custom/tag_reference-1.0.0>
     """
 
     with tag_reference_extension():
-        buff = helpers.yaml_to_asdf(yaml)
+        buff = yaml_to_asdf(yaml)
         with asdf.open(buff) as ff:
             custom = ff.tree["custom"]
             assert custom.name == "Something"
@@ -814,16 +820,16 @@ custom: !<tag:nowhere.org:custom/foreign_tag_reference-1.0.0>
         cfg.add_resource_mapping({schema_uri: tag_schema})
         cfg.add_extension(ForeignTagReferenceExtension())
 
-        buff = helpers.yaml_to_asdf(yaml)
+        buff = yaml_to_asdf(yaml)
         with asdf.open(buff) as ff:
             a = ff.tree["custom"].a
             assert a.name == "Something"
             assert_array_equal(a.things, [1, 2, 3])
 
 
-def test_self_reference_resolution():
+def test_self_reference_resolution(test_data_path):
     s = schema.load_schema(
-        helpers.get_test_data_path("self_referencing-1.0.0.yaml"),
+        test_data_path / "self_referencing-1.0.0.yaml",
         resolve_references=True,
     )
     assert "$ref" not in repr(s)
@@ -832,38 +838,32 @@ def test_self_reference_resolution():
 
 def test_schema_resolved_via_entry_points():
     """Test that entry points mappings to core schema works"""
-    tag = asdf.testing.helpers.format_tag("stsci.edu", "asdf", "1.0.0", "fits/fits")
+    tag = "tag:stsci.edu:asdf/fits/fits-1.0.0"
     extension_manager = asdf.extension.get_cached_extension_manager(get_config().extensions)
     schema_uris = extension_manager.get_tag_definition(tag).schema_uris
     assert len(schema_uris) > 0
-    s = schema.load_schema(schema_uris[0], resolve_references=True)
-    assert tag in repr(s)
+    s = schema.load_schema(schema_uris[0])
+    assert s["id"] == schema_uris[0]
 
 
 @pytest.mark.parametrize("num", [constants.MAX_NUMBER + 1, constants.MIN_NUMBER - 1])
 def test_max_min_literals(num):
     msg = r"Integer value .* is too large to safely represent as a literal in ASDF"
 
-    tree = {
-        "test_int": num,
-    }
-
+    af = asdf.AsdfFile()
+    af["test_int"] = num
     with pytest.raises(ValidationError, match=msg):
-        asdf.AsdfFile(tree)
-
-    tree = {
-        "test_list": [num],
-    }
+        af.validate()
 
+    af = asdf.AsdfFile()
+    af["test_list"] = [num]
     with pytest.raises(ValidationError, match=msg):
-        asdf.AsdfFile(tree)
-
-    tree = {
-        num: "test_key",
-    }
+        af.validate()
 
+    af = asdf.AsdfFile()
+    af[num] = "test_key"
     with pytest.raises(ValidationError, match=msg):
-        asdf.AsdfFile(tree)
+        af.validate()
 
 
 @pytest.mark.parametrize("num", [constants.MAX_NUMBER + 1, constants.MIN_NUMBER - 1])
@@ -891,14 +891,14 @@ def test_max_min_literals_write(num, ttype, tmp_path):
 def test_read_large_literal(value):
     yaml = f"integer: {value}"
 
-    buff = helpers.yaml_to_asdf(yaml)
+    buff = yaml_to_asdf(yaml)
 
     with pytest.warns(AsdfWarning, match=r"Invalid integer literal value"), asdf.open(buff) as af:
         assert af["integer"] == value
 
     yaml = f"{value}: foo"
 
-    buff = helpers.yaml_to_asdf(yaml)
+    buff = yaml_to_asdf(yaml)
 
     with pytest.warns(AsdfWarning, match=r"Invalid integer literal value"), asdf.open(buff) as af:
         assert af[value] == "foo"
@@ -913,13 +913,12 @@ def test_read_large_literal(value):
 )
 def test_mapping_supported_key_types(keys, version):
     for key in keys:
-        with helpers.assert_no_warnings():
-            af = asdf.AsdfFile({key: "value"}, version=version)
-            buff = io.BytesIO()
-            af.write_to(buff)
-            buff.seek(0)
-            with asdf.open(buff) as af:
-                assert af[key] == "value"
+        af = asdf.AsdfFile({key: "value"}, version=version)
+        buff = io.BytesIO()
+        af.write_to(buff)
+        buff.seek(0)
+        with asdf.open(buff) as af:
+            assert af[key] == "value"
 
 
 @pytest.mark.parametrize(
@@ -930,8 +929,10 @@ def test_mapping_supported_key_types(keys, version):
 )
 def test_mapping_unsupported_key_types(keys, version):
     for key in keys:
+        af = asdf.AsdfFile(version=version)
+        af[key] = "value"
         with pytest.raises(ValidationError, match=r"Mapping key .* is not permitted"):
-            asdf.AsdfFile({key: "value"}, version=version)
+            af.validate()
 
 
 def test_nested_array():
@@ -1010,8 +1011,8 @@ properties:
             schema.validate(b, schema=schema_tree)
 
 
-def test_custom_validation_bad(tmp_path):
-    custom_schema_path = helpers.get_test_data_path("custom_schema.yaml")
+def test_custom_validation_bad(tmp_path, test_data_path):
+    custom_schema_path = test_data_path / "custom_schema.yaml"
     asdf_file = str(tmp_path / "out.asdf")
 
     # This tree does not conform to the custom schema
@@ -1022,10 +1023,10 @@ def test_custom_validation_bad(tmp_path):
         ff.write_to(asdf_file)
 
     # Creating file using custom schema should fail
-    with pytest.raises(ValidationError, match=r".* is a required property"), asdf.AsdfFile(
-        tree,
-        custom_schema=custom_schema_path,
-    ):
+    af = asdf.AsdfFile(custom_schema=custom_schema_path)
+    af._tree = asdf.tags.core.AsdfObject(tree)
+    with pytest.raises(ValidationError, match=r".* is a required property"):
+        af.validate()
         pass
 
     # Opening file without custom schema should pass
@@ -1033,15 +1034,18 @@ def test_custom_validation_bad(tmp_path):
         pass
 
     # Opening file with custom schema should fail
-    with pytest.raises(ValidationError, match=r".* is a required property"), asdf.open(
-        asdf_file,
-        custom_schema=custom_schema_path,
+    with (
+        pytest.raises(ValidationError, match=r".* is a required property"),
+        asdf.open(
+            asdf_file,
+            custom_schema=custom_schema_path,
+        ),
     ):
         pass
 
 
-def test_custom_validation_good(tmp_path):
-    custom_schema_path = helpers.get_test_data_path("custom_schema.yaml")
+def test_custom_validation_good(tmp_path, test_data_path):
+    custom_schema_path = test_data_path / "custom_schema.yaml"
     asdf_file = str(tmp_path / "out.asdf")
 
     # This tree conforms to the custom schema
@@ -1054,15 +1058,13 @@ def test_custom_validation_good(tmp_path):
         pass
 
 
-def test_custom_validation_pathlib(tmp_path):
+def test_custom_validation_pathlib(tmp_path, test_data_path):
     """
     Make sure custom schema paths can be pathlib.Path objects
 
     See https://github.com/asdf-format/asdf/issues/653 for discussion.
     """
-    from pathlib import Path
-
-    custom_schema_path = Path(helpers.get_test_data_path("custom_schema.yaml"))
+    custom_schema_path = test_data_path / "custom_schema.yaml"
     asdf_file = str(tmp_path / "out.asdf")
 
     # This tree conforms to the custom schema
@@ -1075,8 +1077,8 @@ def test_custom_validation_pathlib(tmp_path):
         pass
 
 
-def test_custom_validation_with_definitions_good(tmp_path):
-    custom_schema_path = helpers.get_test_data_path("custom_schema_definitions.yaml")
+def test_custom_validation_with_definitions_good(tmp_path, test_data_path):
+    custom_schema_path = test_data_path / "custom_schema_definitions.yaml"
     asdf_file = str(tmp_path / "out.asdf")
 
     # This tree conforms to the custom schema
@@ -1089,8 +1091,8 @@ def test_custom_validation_with_definitions_good(tmp_path):
         pass
 
 
-def test_custom_validation_with_definitions_bad(tmp_path):
-    custom_schema_path = helpers.get_test_data_path("custom_schema_definitions.yaml")
+def test_custom_validation_with_definitions_bad(tmp_path, test_data_path):
+    custom_schema_path = test_data_path / "custom_schema_definitions.yaml"
     asdf_file = str(tmp_path / "out.asdf")
 
     # This tree does NOT conform to the custom schema
@@ -1101,26 +1103,28 @@ def test_custom_validation_with_definitions_bad(tmp_path):
         ff.write_to(asdf_file)
 
     # Creating file with custom schema should fail
-    with pytest.raises(ValidationError, match=r".* is a required property"), asdf.AsdfFile(
-        tree,
-        custom_schema=custom_schema_path,
-    ):
-        pass
+    af = asdf.AsdfFile(custom_schema=custom_schema_path)
+    af._tree = asdf.tags.core.AsdfObject(tree)
+    with pytest.raises(ValidationError, match=r".* is a required property"):
+        af.validate()
 
     # Opening file without custom schema should pass
     with asdf.open(asdf_file):
         pass
 
     # Opening file with custom schema should fail
-    with pytest.raises(ValidationError, match=r".* is a required property"), asdf.open(
-        asdf_file,
-        custom_schema=custom_schema_path,
+    with (
+        pytest.raises(ValidationError, match=r".* is a required property"),
+        asdf.open(
+            asdf_file,
+            custom_schema=custom_schema_path,
+        ),
     ):
         pass
 
 
-def test_custom_validation_with_external_ref_good(tmp_path):
-    custom_schema_path = helpers.get_test_data_path("custom_schema_external_ref.yaml")
+def test_custom_validation_with_external_ref_good(tmp_path, test_data_path):
+    custom_schema_path = test_data_path / "custom_schema_external_ref.yaml"
     asdf_file = str(tmp_path / "out.asdf")
 
     # This tree conforms to the custom schema
@@ -1133,8 +1137,8 @@ def test_custom_validation_with_external_ref_good(tmp_path):
         pass
 
 
-def test_custom_validation_with_external_ref_bad(tmp_path):
-    custom_schema_path = helpers.get_test_data_path("custom_schema_external_ref.yaml")
+def test_custom_validation_with_external_ref_bad(tmp_path, test_data_path):
+    custom_schema_path = test_data_path / "custom_schema_external_ref.yaml"
     asdf_file = str(tmp_path / "out.asdf")
 
     # This tree does not conform to the custom schema
@@ -1145,20 +1149,22 @@ def test_custom_validation_with_external_ref_bad(tmp_path):
         ff.write_to(asdf_file)
 
     # Creating file with custom schema should fail
-    with pytest.raises(ValidationError, match=r"False is not valid under any of the given schemas"), asdf.AsdfFile(
-        tree,
-        custom_schema=custom_schema_path,
-    ):
-        pass
+    af = asdf.AsdfFile(custom_schema=custom_schema_path)
+    af["foo"] = False
+    with pytest.raises(ValidationError, match=r"False is not valid under any of the given schemas"):
+        af.validate()
 
     # Opening file without custom schema should pass
     with asdf.open(asdf_file):
         pass
 
     # Opening file with custom schema should fail
-    with pytest.raises(ValidationError, match=r"False is not valid under any of the given schemas"), asdf.open(
-        asdf_file,
-        custom_schema=custom_schema_path,
+    with (
+        pytest.raises(ValidationError, match=r"False is not valid under any of the given schemas"),
+        asdf.open(
+            asdf_file,
+            custom_schema=custom_schema_path,
+        ),
     ):
         pass
 
@@ -1196,10 +1202,10 @@ def test_numpy_scalar_type_validation(numpy_value, valid_types):
 
         if valid is not expected_valid:
             description = "valid" if expected_valid else "invalid"
-            msg = "Expected numpy.{} to be {} against jsonschema type '{}'".format(
-                type(numpy_value).__name__,
-                description,
-                jsonschema_type,
+            msg = (
+                f"Expected numpy.{type(numpy_value).__name__} "
+                f"to be {description} against jsonschema type "
+                f"'{jsonschema_type}'"
             )
             raise AssertionError(msg)
 
